@@ -1,6 +1,7 @@
 import time
 import uuid
-from typing import Optional
+import json
+from spotapi.types.annotations import enforce
 from spotapi.types import Config
 from spotapi.exceptions import GeneratorError
 from spotapi.http.request import TLSClient
@@ -11,30 +12,54 @@ from spotapi.utils.strings import (
     random_dob,
 )
 
+__all__ = ["Creator", "AccountChallenge", "GeneratorError"]
 
+
+@enforce
 class Creator:
     """
     Creates a new Spotify account.
+
+    Parameters
+    ----------
+    cfg (Config): Configuration object.
+    email (str, optional): Email address to use for the account. Defaults to a randomly generated email.
+    display_name (str, optional): Display name to use for the account. Defaults to a randomly generated string.
+    password (str, optional): Password to use for the account. Defaults to a randomly generated string.
     """
 
+    __slots__ = (
+        "email",
+        "password",
+        "display_name",
+        "cfg",
+        "client",
+        "submission_id",
+        "api_key",
+        "installation_id",
+        "csrf_token",
+        "flow_id",
+    )
+
     def __init__(
         self,
         cfg: Config,
-        email: Optional[str] = random_email(),
-        password: Optional[str] = random_string(10, True),
+        email: str = random_email(),
+        display_name: str = random_string(10),
+        password: str = random_string(10, strong=True),
     ) -> None:
         self.email = email
         self.password = password
+        self.display_name = display_name
         self.cfg = cfg
 
         self.client = self.cfg.client
         self.submission_id = str(uuid.uuid4())
 
-    def __get_session(self) -> None:
+    def _get_session(self) -> None:
         url = "https://www.spotify.com/ca-en/signup"
         resp = self.client.get(url)
 
-
         if resp.fail:
             raise GeneratorError("Could not get session", error=resp.error.string)
 
@@ -43,7 +68,7 @@ class Creator:
         self.csrf_token = parse_json_string(resp.response, "csrfToken")
         self.flow_id = parse_json_string(resp.response, "flowId")
 
-    def __process_register(self, captcha_token: str) -> None:
+    def _process_register(self, captcha_token: str) -> None:
         payload = {
             "account_details": {
                 "birthdate": random_dob(),
@@ -52,7 +77,7 @@ class Creator:
                     "send_email": True,
                     "third_party_email": False,
                 },
-                "display_name": "Aran",
+                "display_name": self.display_name,
                 "email_and_password_identifier": {
                     "email": self.email,
                     "password": self.password,
@@ -80,7 +105,6 @@ class Creator:
 
         resp = self.client.post(url, json=payload)
 
-
         if resp.fail:
             raise GeneratorError(
                 "Could not process registration", error=resp.error.string
@@ -91,38 +115,51 @@ class Creator:
             AccountChallenge(self.client, resp.response, self.cfg).defeat_challenge()
 
     def register(self) -> None:
-        self.__get_session()
-        # Pylance keeps complaining about self keyword. I don't know why.
+        self._get_session()
+        if self.cfg.solver is None:
+            raise GeneratorError("Solver not set")
+
         captcha_token = self.cfg.solver.solve_captcha(
             "https://www.spotify.com/ca-en/signup",
             "6LfCVLAUAAAAALFwwRnnCJ12DalriUGbj8FW_J39",
             "website/signup/submit_email",
             "v3",
         )
-        self.__process_register(captcha_token)
+        self._process_register(captcha_token)
 
 
 class AccountChallenge:
+    __slots__ = (
+        "client",
+        "raw",
+        "session_id",
+        "cfg",
+        "challenge_url",
+    )
+
     def __init__(self, client: TLSClient, raw_response: str, cfg: Config) -> None:
         self.client = client
         self.raw = raw_response
-        self.session_id = parse_json_string(raw_response, "session_id")
+        self.session_id = parse_json_string(
+            json.dumps(raw_response, separators=(",", ":")), "session_id"
+        )
         self.cfg = cfg
 
-    def __get_session(self) -> None:
+    def _get_session(self) -> None:
         url = "https://challenge.spotify.com/api/v1/get-session"
         payload = {"session_id": self.session_id}
         resp = self.client.post(url, json=payload)
 
-
         if resp.fail:
             raise GeneratorError(
                 "Could not get challenge session", error=resp.error.string
             )
 
-        self.challenge_url = parse_json_string(resp.response, "url")
+        self.challenge_url = parse_json_string(
+            json.dumps(resp.response, separators=(",", ":")), "url"
+        )
 
-    def __submit_challenge(self, token: str) -> None:
+    def _submit_challenge(self, token: str) -> None:
         session_id = self.challenge_url.split("c/")[1].split("/")[0]
         challenge_id = self.challenge_url.split(session_id + "/")[1].split("/")[0]
         url = "https://challenge.spotify.com/api/v1/invoke-challenge-command"
@@ -139,36 +176,35 @@ class AccountChallenge:
             },
         )
 
-
         if resp.fail:
-            raise GeneratorError(
-                "Could not submit challenge", error=resp.error.string
-            )
+            raise GeneratorError("Could not submit challenge", error=resp.error.string)
 
-    def __complete_challenge(self) -> None:
+    def _complete_challenge(self) -> None:
         url = (
             "https://spclient.wg.spotify.com/signup/public/v2/account/complete-creation"
         )
         payload = {"session_id": self.session_id}
         resp = self.client.post(url, json=payload)
 
-
         if resp.fail:
             raise GeneratorError(
                 "Could not complete challenge", error=resp.error.string
             )
 
-        if not ("success" in resp.response):
+        if "success" not in resp.response:
             raise GeneratorError("Could not complete challenge", error=resp.response)
 
     def defeat_challenge(self) -> None:
-        self.__get_session()
+        self._get_session()
+        if self.cfg.solver is None:
+            raise GeneratorError("Solver not set")
+
         token = self.cfg.solver.solve_captcha(
             self.challenge_url,
             "6LeO36obAAAAALSBZrY6RYM1hcAY7RLvpDDcJLy3",
             "challenge",
             "v2",
         )
-        self.__submit_challenge(token)
-        self.__complete_challenge()
+        self._submit_challenge(token)
+        self._complete_challenge()
         self.cfg.logger.info("Successfully defeated challenge. Account created.")
