@@ -3,43 +3,57 @@ from __future__ import annotations
 import json
 import re
 import time
-from typing import Any, Generator, Mapping, Optional
-
+from typing import Any
+from spotapi.login import Login
+from spotapi.user import User
+from spotapi.client import BaseClient
+from collections.abc import Mapping, Generator
+from spotapi.types.annotations import enforce
 from spotapi.exceptions import PlaylistError
 from spotapi.http.request import TLSClient
-from spotapi.login import Login
-from spotapi.client import BaseClient
-from spotapi.user import User
+
+__all__ = ["PublicPlaylist", "PrivatePlaylist", "PlaylistError"]
 
 
+@enforce
 class PublicPlaylist:
     """
     Allows you to get all public information on a playlist.
     No login is required.
+
+    Parameters
+    ----------
+    playlist (Optional[str]): The Spotify URI of the playlist.
+    client (TLSClient): An instance of TLSClient to use for requests.
     """
 
+    __slots__ = (
+        "base",
+        "playlist_id",
+        "playlist_link",
+    )
+
     def __init__(
         self,
-        playlist: Optional[str] = None,
+        playlist: str,
         /,
         *,
         client: TLSClient = TLSClient("chrome_120", "", auto_retries=3),
     ) -> None:
         self.base = BaseClient(client=client)
-
-        if playlist:
-            self.playlist_id = (
-                playlist.split("playlist/")[-1] if "playlist" in playlist else playlist
-            )
-            self.playlist_link = f"https://open.spotify.com/playlist/{self.playlist_id}"
+        self.playlist_id = (
+            playlist.split("playlist/")[-1] if "playlist" in playlist else playlist
+        )
+        self.playlist_link = f"https://open.spotify.com/playlist/{self.playlist_id}"
 
     def get_playlist_info(
-        self, limit: Optional[int] = 25, *, offset: Optional[int] = 0
+        self,
+        limit: int = 25,
+        *,
+        offset: int = 0,
+        enable_watch_feed_entrypoint: bool = False,
     ) -> Mapping[str, Any]:
         """Gets the public playlist information"""
-        if not self.playlist_id:
-            raise ValueError("Playlist ID not set")
-
         url = "https://api-partner.spotify.com/pathfinder/v1/query"
         params = {
             "operationName": "fetchPlaylist",
@@ -48,6 +62,7 @@ class PublicPlaylist:
                     "uri": f"spotify:playlist:{self.playlist_id}",
                     "offset": offset,
                     "limit": limit,
+                    "enableWatchFeedEntrypoint": enable_watch_feed_entrypoint,
                 }
             ),
             "extensions": json.dumps(
@@ -74,7 +89,7 @@ class PublicPlaylist:
         """
         Generator that fetches playlist information in chunks
 
-        Note: If total_tracks <= 343, then there is no need to paginate
+        NOTE: If total_tracks <= 343, then there is no need to paginate.
         """
         UPPER_LIMIT: int = 343
         # We need to get the total playlists first
@@ -97,12 +112,25 @@ class PublicPlaylist:
 class PrivatePlaylist:
     """
     Methods on playlists that you can only do whilst logged in.
+
+    Parameters
+    ----------
+    login (Login): The login object to use
+    playlist (Optional[str]): The Spotify URI of the playlist.
     """
 
+    __slots__ = (
+        "base",
+        "login",
+        "user",
+        "_playlist",
+        "playlist_id",
+    )
+
     def __init__(
         self,
         login: Login,
-        playlist: Optional[str] = None,
+        playlist: str | None = None,
     ) -> None:
         if not login.logged_in:
             raise ValueError("Must be logged in")
@@ -122,8 +150,8 @@ class PrivatePlaylist:
         if "playlist:" in playlist:
             playlist = playlist.split("playlist:")[-1]
 
-        if hasattr(playlist, "playlist_id"):
-            self.playlist_id = playlist
+        if not playlist:
+            raise ValueError("Playlist not set")
 
         setattr(self, "playlist_id", playlist)
         self._playlist = True
@@ -165,7 +193,6 @@ class PrivatePlaylist:
 
         resp = self.login.client.post(url, json=payload, authenticate=True)
 
-
         if resp.fail:
             raise PlaylistError(
                 "Could not add playlist to library", error=resp.error.string
@@ -211,7 +238,7 @@ class PrivatePlaylist:
         # They are the same requests
         return self.remove_from_library()
 
-    def get_library(self, limit: Optional[int] = 50) -> Mapping[str, Any]:
+    def get_library(self, limit: int = 50, /) -> Mapping[str, Any]:
         """Gets all the playlists in your library"""
         url = "https://api-partner.spotify.com/pathfinder/v1/query"
         params = {
@@ -247,7 +274,7 @@ class PrivatePlaylist:
 
         return resp.response
 
-    def __stage_create_playlist(self, name: str) -> str:
+    def _stage_create_playlist(self, name: str) -> str:
         url = "https://spclient.wg.spotify.com/playlist/v2/playlist"
         payload = {
             "ops": [
@@ -284,7 +311,7 @@ class PrivatePlaylist:
 
     def create_playlist(self, name: str) -> str:
         """Creates a new playlist"""
-        playlist_id = self.__stage_create_playlist(name)
+        playlist_id = self._stage_create_playlist(name)
         url = f"https://spclient.wg.spotify.com/playlist/v2/user/{self.user.username}/rootlist/changes"
         payload = {
             "deltas": [
@@ -321,3 +348,20 @@ class PrivatePlaylist:
             raise PlaylistError("Could not create playlist", error=resp.error.string)
 
         return playlist_id
+
+    def recommended_songs(self, num_songs: int = 20) -> Mapping[str, Any]:
+        """Gets the recommended songs for the playlist"""
+        url = "https://spclient.wg.spotify.com/playlistextender/extendp/"
+        payload = {
+            "playlistURI": f"spotify:playlist:{self.playlist_id}",
+            "trackSkipIDs": [],
+            "numResults": num_songs,
+        }
+        resp = self.login.client.post(url, json=payload, authenticate=True)
+
+        if resp.fail:
+            raise PlaylistError(
+                "Could not get recommended songs", error=resp.error.string
+            )
+
+        return resp.response
