@@ -1,32 +1,79 @@
 import json
-from typing import Any, Generator, List, Mapping, Optional, Tuple
-
+from typing import Any, List, Tuple
+from spotapi.types.annotations import enforce
 from spotapi.exceptions import SongError
 from spotapi.http.request import TLSClient
 from spotapi.client import BaseClient
+from collections.abc import Mapping, Iterable, Generator
 from spotapi.playlist import PrivatePlaylist, PublicPlaylist
 
+__all__ = ["Song", "SongError"]
 
+
+@enforce
 class Song:
     """
     Extends the PrivatePlaylist class with methods that can only be used while logged in.
     These methods interact with songs and tend to be used in the context of a playlist.
+
+    Parameters
+    ----------
+    playlist (Optional[str]): The Spotify URI of the playlist.
+    client (Optional[TLSClient]): An instance of TLSClient to use for requests
     """
 
+    __slots__ = (
+        "playlist",
+        "base",
+    )
+
     def __init__(
         self,
-        playlist: Optional[PrivatePlaylist] = None,
+        playlist: PrivatePlaylist | None = None,
         *,
-        client: Optional[TLSClient] = TLSClient("chrome_120", "", auto_retries=3),
+        client: TLSClient = TLSClient("chrome_120", "", auto_retries=3),
     ) -> None:
         self.playlist = playlist
-        self.base = BaseClient(client=playlist.client if (playlist is not None) else client) # type: ignore
+        self.base = BaseClient(client=playlist.login.client if playlist else client)
+
+    def get_track_info(self, track_id: str) -> Mapping[str, Any]:
+        """
+        Gets information about a specific song.
+        """
+        url = "https://api-partner.spotify.com/pathfinder/v1/query"
+        params = {
+            "operationName": "getTrack",
+            "variables": json.dumps(
+                {
+                    "uri": f"spotify:track:{track_id}",
+                }
+            ),
+            "extensions": json.dumps(
+                {
+                    "persistedQuery": {
+                        "version": 1,
+                        "sha256Hash": self.base.part_hash("getTrack"),
+                    }
+                }
+            ),
+        }
+
+        resp = self.base.client.post(url, params=params, authenticate=True)
+
+        if resp.fail:
+            raise SongError("Could not get song info", error=resp.error.string)
+
+        if not isinstance(resp.response, Mapping):
+            raise SongError("Invalid JSON")
+
+        return resp.response
 
     def query_songs(
-        self, query: str, /, limit: Optional[int] = 10, *, offset: Optional[int] = 0
+        self, query: str, /, limit: int = 10, *, offset: int = 0
     ) -> Mapping[str, Any]:
         """
         Searches for songs in the Spotify catalog.
+        NOTE: Returns the raw result unlike paginate_songs which only returns the songs.
         """
         url = "https://api-partner.spotify.com/pathfinder/v1/query"
         params = {
@@ -67,7 +114,7 @@ class Song:
         """
         Generator that fetches songs in chunks
 
-        Note: If total_tracks <= 100, then there is no need to paginate
+        Note: If total_count <= 100, then there is no need to paginate
         """
         UPPER_LIMIT: int = 100
         # We need to get the total songs first
@@ -86,18 +133,16 @@ class Song:
             ]["tracksV2"]["items"]
             offset += UPPER_LIMIT
 
-    def add_song_to_playlist(self, song_id: str, /) -> None:
-        """Adds a song to the playlist"""
+    def add_songs_to_playlist(self, song_ids: List[str], /) -> None:
+        """Adds multiple songs to the playlist"""
+        # This can be a bit slow when adding 500+ songs, maybe we should add a batch processing
         if not self.playlist or not hasattr(self.playlist, "playlist_id"):
             raise ValueError("Playlist not set")
 
-        if "track" in song_id:
-            song_id = song_id.split("track/")[1]
-
         url = "https://api-partner.spotify.com/pathfinder/v1/query"
         payload = {
             "variables": {
-                "uris": [f"spotify:track:{song_id}"],
+                "uris": [f"spotify:track:{song_id}" for song_id in song_ids],
                 "playlistUri": f"spotify:playlist:{self.playlist.playlist_id}",
                 "newPosition": {"moveType": "BOTTOM_OF_PLAYLIST", "fromUid": None},
             },
@@ -112,12 +157,19 @@ class Song:
         resp = self.base.client.post(url, json=payload, authenticate=True)
 
         if resp.fail:
-            raise SongError("Could not add song to playlist", error=resp.error.string)
+            raise SongError("Could not add songs to playlist", error=resp.error.string)
 
-    def __stage_remove_song(self, uids: List[str]) -> None:
+    def add_song_to_playlist(self, song_id: str, /) -> None:
+        """Adds a song to the playlist"""
+        if "track" in song_id:
+            song_id = song_id.split("track/")[1]
+
+        self.add_songs_to_playlist([song_id])
+
+    def _stage_remove_song(self, uids: List[str]) -> None:
         # If None, something internal went wrong
         assert self.playlist is not None, "Playlist not set"
-        
+
         url = "https://api-partner.spotify.com/pathfinder/v1/query"
         payload = {
             "variables": {
@@ -140,12 +192,13 @@ class Song:
                 "Could not remove song from playlist", error=resp.error.string
             )
 
-    def __parse_playlist_items(
-        self,
-        items: List[Mapping[str, Any]],
-        song_id: Optional[str] = None,
-        song_name: Optional[str] = None,
-        all_instances: Optional[bool] = False,
+    @staticmethod
+    def parse_playlist_items(
+        items: Iterable[Mapping[str, Any]],
+        *,
+        song_id: str | None = None,
+        song_name: str | None = None,
+        all_instances: bool = False,
     ) -> Tuple[List[str], bool]:
         uids: List[str] = []
         for item in items:
@@ -166,10 +219,10 @@ class Song:
     def remove_song_from_playlist(
         self,
         *,
-        uid: Optional[str] = None,
-        song_id: Optional[str] = None,
-        song_name: Optional[str] = None,
-        all_instances: Optional[bool] = False,
+        all_instances: bool = False,
+        uid: str | None = None,
+        song_id: str | None = None,
+        song_name: str | None = None,
     ) -> None:
         """
         Removes a song from the playlist.
@@ -193,7 +246,7 @@ class Song:
         if not uid:
             for playlist_chunk in playlist:
                 items = playlist_chunk["items"]
-                extended_uids, stop = self.__parse_playlist_items(
+                extended_uids, stop = Song.parse_playlist_items(
                     items,
                     song_id=song_id,
                     song_name=song_name,
@@ -210,7 +263,7 @@ class Song:
         if len(uids) == 0:
             raise SongError("Song not found in playlist")
 
-        self.__stage_remove_song(uids)
+        self._stage_remove_song(uids)
 
     def like_song(self, song_id: str, /) -> None:
         if not self.playlist or not hasattr(self.playlist, "playlist_id"):
