@@ -1,20 +1,32 @@
 from __future__ import annotations
 
-from typing import Any, Mapping
-
+from typing import Any
+from collections.abc import Mapping
+from spotapi.types.annotations import enforce
+from spotapi import utils
 from spotapi.exceptions import UserError
 from spotapi.login import Login
 
+__all__ = ["User", "UserError"]
 
+
+@enforce
 class User:
     """
     Represents a Spotify user.
+
+    Parameters
+    ----------
+    login : Login
+        The login object.
     """
 
+    __slots__ = ("login", "_user_plan", "_user_info", "csrf_token")
+
     def __init__(self, login: Login) -> None:
         if not login.logged_in:
             raise ValueError("Must be logged in")
-        
+
         self.login = login
         self._user_plan: Mapping[str, Any] | None = None
         self._user_info: Mapping[str, Any] | None = None
@@ -29,7 +41,7 @@ class User:
     @property
     def username(self) -> str:
         if self._user_info is None:
-           self._user_info = self.get_user_info()
+            self._user_info = self.get_user_info()
 
         return self._user_info["profile"]["username"]
 
@@ -38,7 +50,6 @@ class User:
         url = "https://www.spotify.com/ca-en/api/account/v2/plan/"
         resp = self.login.client.get(url)
 
-
         if resp.fail:
             raise UserError("Could not get user plan info", error=resp.error.string)
 
@@ -56,14 +67,12 @@ class User:
             raise e
         else:
             return True
-        
 
     def get_user_info(self) -> Mapping[str, Any]:
         """Gets accounts user info."""
         url = "https://www.spotify.com/api/account-settings/v1/profile"
         resp = self.login.client.get(url)
 
-
         if resp.fail:
             raise UserError("Could not get user info", error=resp.error.string)
 
@@ -79,6 +88,9 @@ class User:
         For this function to work, dump must be the entire profile dump.
         You can get this dump from get_user_info, then change the fields you want.
         """
+        if self.login.solver is None:
+            raise UserError("Captcha solver not set")
+
         captcha_response = self.login.solver.solve_captcha(
             "https://www.spotify.com",
             "6LfCVLAUAAAAALFwwRnnCJ12DalriUGbj8FW_J39",
@@ -98,9 +110,12 @@ class User:
                 "country": profile_dump["country"],
             },
             "recaptcha_token": captcha_response,
+            "client_nonce": utils.random_nonce(),
+            "callback_url": "https://www.spotify.com/account/profile/challenge",
+            "client_info": {"locale": "en_US", "capabilities": [1]},
         }
 
-        url = "https://www.spotify.com/api/account-settings/v1/profile"
+        url = "https://www.spotify.com/api/account-settings/v2/profile"
 
         headers = {
             "Content-Type": "application/json",
@@ -109,6 +124,5 @@ class User:
 
         resp = self.login.client.put(url, json=dump, headers=headers)
 
-
         if resp.fail:
             raise UserError("Could not edit  user info", error=resp.error.string)
