@@ -1,10 +1,13 @@
 import os
 import time
+from typing import Any
 from threading import Lock
 from datetime import datetime
 from colorama import Fore, Style, init
 from spotapi.types import LoggerProtocol
 
+__all__ = ["Logger", "NoopLogger", "LoggerProtocol"]
+
 os.system("")
 init(autoreset=True)
 
@@ -24,7 +27,7 @@ class Logger(LoggerProtocol):
         return f"[{Style.BRIGHT}{Fore.LIGHTCYAN_EX}{str(t)}{Style.RESET_ALL}]"
 
     @staticmethod
-    def error(s: str, **extra) -> None:
+    def error(s: str, **extra: Any) -> None:
         with LOCK:
             fields = [
                 f"{Style.BRIGHT}{Fore.LIGHTBLUE_EX}{k}={Fore.LIGHTRED_EX}{v}{Style.RESET_ALL}"
@@ -36,7 +39,7 @@ class Logger(LoggerProtocol):
             )
 
     @staticmethod
-    def debug(s: str, **extra) -> None:
+    def attempt(s: str, **extra: Any) -> None:
         with LOCK:
             fields = [
                 f"{Style.BRIGHT}{Fore.LIGHTBLUE_EX}{k}={Fore.LIGHTYELLOW_EX}{v}{Style.RESET_ALL}"
@@ -48,7 +51,7 @@ class Logger(LoggerProtocol):
             )
 
     @staticmethod
-    def info(s: str, **extra) -> None:
+    def info(s: str, **extra: Any) -> None:
         with LOCK:
             fields = [
                 f"{Style.BRIGHT}{Fore.LIGHTBLUE_EX}{k}={Fore.LIGHTMAGENTA_EX}{v}{Style.RESET_ALL}"
@@ -60,7 +63,7 @@ class Logger(LoggerProtocol):
             )
 
     @staticmethod
-    def fatal(s: str, **extra) -> None:
+    def fatal(s: str, **extra: Any) -> None:
         with LOCK:
             fields = [
                 f"{Style.BRIGHT}{Fore.LIGHTBLUE_EX}{k}={Fore.LIGHTRED_EX}{v}{Style.RESET_ALL}"
@@ -76,17 +79,17 @@ class Logger(LoggerProtocol):
 
 class NoopLogger(LoggerProtocol):
     @staticmethod
-    def error(s: str, **extra) -> None:
+    def error(s: str, **extra: Any) -> None:
         ...
 
     @staticmethod
-    def info(s: str, **extra) -> None:
+    def info(s: str, **extra: Any) -> None:
         ...
 
     @staticmethod
-    def fatal(s: str, **extra) -> None:
+    def fatal(s: str, **extra: Any) -> None:
         ...
 
     @staticmethod
-    def attempt(s: str, **extra) -> None:
+    def attempt(s: str, **extra: Any) -> None:
         ...
