@@ -7,7 +7,7 @@ import atexit
 import json
 import os
 import sqlite3
-from typing import Any, List, Mapping, Optional
+from typing import Any, List, Mapping
 
 import pymongo
 import redis
@@ -16,12 +16,21 @@ from readerwriterlock import rwlock
 from spotapi.types.interfaces import SaverProtocol
 from spotapi.exceptions import SaverError
 
+__all__ = ["JSONSaver", "MongoSaver", "RedisSaver", "SqliteSaver", "SaverProtocol"]
+
 
 class JSONSaver(SaverProtocol):
     """
     CRUD methods for JSON files
     """
 
+    __slots__ = (
+        "path",
+        "rwlock",
+        "rlock",
+        "wlock",
+    )
+
     def __init__(self, path: str = "sessions.json") -> None:
         self.path = path
 
@@ -29,6 +38,9 @@ class JSONSaver(SaverProtocol):
         self.rlock = self.rwlock.gen_rlock()
         self.wlock = self.rwlock.gen_wlock()
 
+    def __str__(self) -> str:
+        return f"JSONSaver()"
+
     def save(self, data: List[Mapping[str, Any]], **kwargs) -> None:
         """
         Save data to a JSON file
@@ -51,6 +63,15 @@ class JSONSaver(SaverProtocol):
                 with open(self.path, "r") as f:
                     file_content = f.read()
                     current = json.loads(file_content) if file_content.strip() else []
+                    # Checks if identifier exists
+                    if current:
+                        current = [
+                            item
+                            for item in current
+                            if not any(
+                                item["identifier"] == d["identifier"] for d in data
+                            )
+                        ]
 
             current.extend(data)
 
@@ -132,6 +153,15 @@ class SqliteSaver(SaverProtocol):
     CRUD methods for SQLite3 files
     """
 
+    __slots__ = (
+        "path",
+        "conn",
+        "cursor",
+        "rwlock",
+        "rlock",
+        "wlock",
+    )
+
     def __init__(self, path: str = "sessions.db") -> None:
         self.path = path
         self.conn = sqlite3.connect(self.path, check_same_thread=False)
@@ -157,6 +187,9 @@ class SqliteSaver(SaverProtocol):
         self.rlock = self.rwlock.gen_rlock()
         self.wlock = self.rwlock.gen_wlock()
 
+    def __str__(self) -> str:
+        return f"SqliteSaver()"
+
     def save(self, data: List[Mapping[str, Any]], **kwargs) -> None:
         """
         Saves data to a SQLite3 database
@@ -240,6 +273,12 @@ class MongoSaver(SaverProtocol):
     CRUD methods for MongoDB
     """
 
+    __slots__ = (
+        "conn",
+        "database",
+        "collection",
+    )
+
     def __init__(
         self,
         host: str = "mongodb://localhost:27017/",
@@ -252,6 +291,9 @@ class MongoSaver(SaverProtocol):
 
         atexit.register(self.conn.close)
 
+    def __str__(self) -> str:
+        return f"MongoSaver()"
+
     def save(self, data: List[Mapping[str, Any]], **kwargs) -> None:
         if len(data) == 0:
             raise ValueError("No data to save")
@@ -277,12 +319,13 @@ class MongoSaver(SaverProtocol):
 
 
 class RedisSaver(SaverProtocol):
-    def __init__(
-        self, host: str = "localhost", port: int = 6379, db: int = 0
-    ) -> None:
+    def __init__(self, host: str = "localhost", port: int = 6379, db: int = 0) -> None:
         self.client = redis.StrictRedis(host=host, port=port, db=db)
         atexit.register(self.client.close)
 
+    def __str__(self) -> str:
+        return f"RedisSaver()"
+
     def save(self, data: List[Mapping[str, Any]], **kwargs) -> None:
         if len(data) == 0:
             raise ValueError("No data to save")
